package cn.zgfxrc.boot.common.config;

import io.swagger.annotations.Api;
import java.lang.reflect.Field;
import java.util.List;
import java.util.Objects;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.lang.NonNull;
import org.springframework.util.ReflectionUtils;
import org.springframework.web.servlet.mvc.method.RequestMappingInfoHandlerMapping;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.oas.annotations.EnableOpenApi;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.spring.web.plugins.WebMvcRequestHandlerProvider;

/**
 * Swagger2Config
 *
 * <p><a href="http://127.0.0.1:port/swagger-ui/index.html">访问地址</a>
 *
 * @author huanghs
 */
@EnableOpenApi
@Configuration
public class Swagger2Config {
  @Value("${spring.profiles.active}")
  private String profilesActive;

  @Bean
  public static BeanPostProcessor springfoxHandlerProviderBeanPostProcessor() {
    return new BeanPostProcessor() {

      @Override
      public Object postProcessAfterInitialization(@NonNull Object bean, @NonNull String beanName)
          throws BeansException {
        if (bean instanceof WebMvcRequestHandlerProvider) {
          customizeSpringfoxHandlerMappings(getHandlerMappings(bean));
        }
        return bean;
      }

      private <T extends RequestMappingInfoHandlerMapping> void customizeSpringfoxHandlerMappings(
          List<T> mappings) {
        List<T> copy =
            mappings.stream().filter(mapping -> mapping.getPatternParser() == null).toList();
        mappings.clear();
        mappings.addAll(copy);
      }

      @SuppressWarnings("unchecked")
      private List<RequestMappingInfoHandlerMapping> getHandlerMappings(Object bean) {
        try {
          Field field = ReflectionUtils.findField(bean.getClass(), "handlerMappings");
          Objects.requireNonNull(field).setAccessible(true);
          return (List<RequestMappingInfoHandlerMapping>) field.get(bean);
        } catch (IllegalArgumentException | IllegalAccessException | NullPointerException e) {
          throw new IllegalStateException(e);
        }
      }
    };
  }

  // 生成接口信息，包括标题、联系人，联系方式等
  private ApiInfo apiInfo() {
    return new ApiInfoBuilder()
        .title("Swagger3接口文档")
        .description("如有疑问，请联系开发工程师")
        .contact(new Contact("huanghs", "https://seepine.com/", "huanghs@zgfxrc.cn"))
        .version("1.0")
        .build();
  }

  @Bean
  public Docket defaultDocket() {
    // swagger设置，基本信息，要解析的接口及路径等
    return new Docket(DocumentationType.OAS_30)
        .enable(profilesActive.contains("dev"))
        .apiInfo(apiInfo())
        .groupName("defaultDocket")
        .select()
        // 设置通过什么方式定位需要自动生成文档的接口，这里定位方法上的@Api注解
        .apis(RequestHandlerSelectors.withClassAnnotation(Api.class))
        // 接口URI路径设置，any是全路径，也可以通过PathSelectors.regex()正则匹配
        .paths(PathSelectors.any())
        .build();
  }
}
