package cn.zgfxrc.boot.common.config.auth;

import cn.zgfxrc.boot.upms.api.feign.RemoteUserService;
import com.seepine.secret.entity.AuthUser;
import com.seepine.secret.exception.SecretException;
import com.seepine.secret.interfaces.PermissionService;
import com.seepine.tool.cache.Cache;
import com.seepine.tool.lock.Lock;
import com.seepine.tool.secure.digest.MD5;
import java.time.Duration;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nonnull;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Component;

@Slf4j
@Component
@AllArgsConstructor
public class PermissionServiceImpl implements PermissionService {
  private final RemoteUserService remoteUserService;

  @Nonnull
  @Override
  public Set<String> query(@NotNull AuthUser authUser) throws SecretException {
    try {
      return Objects.requireNonNullElse(
          Cache.getIfPresent(
              "secret_permissions:" + MD5.digestHex(authUser.getToken()),
              () -> {
                var permissions =
                    Lock.sync(authUser.getId(), () -> remoteUserService.getPermissions(authUser));
                return permissions.isEmpty() ? null : permissions;
              },
              Duration.ofMinutes(10).toMillis()),
          new HashSet<>());
    } catch (Exception e) {
      throw new SecretException(e);
    }
  }
}
