package cn.zgfxrc.boot.common.config.exception;

import com.seepine.tool.R;
import java.util.Map;
import java.util.Objects;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.Nullable;
import org.springframework.boot.autoconfigure.web.servlet.error.AbstractErrorController;
import org.springframework.boot.web.error.ErrorAttributeOptions;
import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 解决如404等内部错误
 *
 * @author seepine
 */
@Slf4j
@RestController
public class GlobalExceptionController extends AbstractErrorController {
  private static final String ERROR_PATH = "/error";

  public GlobalExceptionController(@Nullable ErrorAttributes errorAttributes) {
    super(Objects.requireNonNull(errorAttributes));
  }

  @RequestMapping(value = ERROR_PATH)
  public R<?> error(HttpServletRequest request, HttpServletResponse response) {
    Map<String, Object> attributes = getErrorAttributes(request, ErrorAttributeOptions.defaults());
    Object status = attributes.get("status");
    String error = attributes.get("error").toString();
    if (status.equals(HttpStatus.NOT_FOUND.value())) {
      error = "Not Found Api Path";
    }
    return R.fail(error);
  }
}
