package cn.zgfxrc.boot.common.config.exception;

import cn.zgfxrc.boot.common.core.util.TenantUtil;
import cn.zgfxrc.boot.common.feign.FeignWrapException;
import com.seepine.secret.exception.ExpiresSecretException;
import com.seepine.secret.exception.ForbiddenSecretException;
import com.seepine.secret.exception.SecretException;
import com.seepine.secret.exception.UnauthorizedSecretException;
import com.seepine.tool.R;
import com.seepine.tool.exception.RunException;
import java.net.UnknownHostException;
import java.sql.SQLSyntaxErrorException;
import javax.servlet.http.HttpServletResponse;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.jdbc.BadSqlGrammarException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * @author Seepine
 */
@Slf4j
@RestControllerAdvice
@AllArgsConstructor
public class GlobalExceptionHandler {

  @ExceptionHandler(UnknownHostException.class)
  @ResponseStatus(HttpStatus.BAD_REQUEST)
  public Object unknownHostException(UnknownHostException e) {
    return R.fail("服务器卡住啦,请五秒后再试试");
  }

  @ExceptionHandler(SecretException.class)
  public R<?> authException(SecretException e, HttpServletResponse response) {
    if (e instanceof ForbiddenSecretException) {
      response.setStatus(HttpStatus.FORBIDDEN.value());
      return R.build(HttpStatus.FORBIDDEN.value(), e.getMessage(), "权限不足");
    } else if (e instanceof UnauthorizedSecretException) {
      response.setStatus(HttpStatus.UNAUTHORIZED.value());
      return R.build(HttpStatus.UNAUTHORIZED.value(), e.getMessage(), "请先登录");
    } else if (e instanceof ExpiresSecretException) {
      response.setStatus(HttpStatus.UNAUTHORIZED.value());
      return R.build(HttpStatus.UNAUTHORIZED.value(), e.getMessage(), "登录过期，请重新登录");
    }
    return R.fail(e.getMessage());
  }

  @ExceptionHandler(IllegalArgumentException.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object illegalArgumentException(IllegalArgumentException e) {
    return R.fail(e.getMessage());
  }

  @ExceptionHandler(TenantUtil.TenantBrokerException.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object tenantBrokerException(TenantUtil.TenantBrokerException e) {
    return R.fail(e.getMessage());
  }

  @ExceptionHandler(RunException.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object runException(RunException e) {
    return R.fail(e.getMessage());
  }

  @ExceptionHandler(FeignWrapException.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object feignWrapException(FeignWrapException e) {
    return R.fail(e.getMessage());
  }

  @ExceptionHandler(Exception.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object exception(Exception e) {
    e.printStackTrace();
    return R.fail("系统内部错误,请联系管理员");
  }

  @ExceptionHandler(DataIntegrityViolationException.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object dataIntegrityViolationException(DataIntegrityViolationException e) {
    log.error("数据库错误!!!!");
    e.printStackTrace();
    return R.fail("系统发生错误！请联系管理员！");
  }

  @ExceptionHandler(BadSqlGrammarException.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object badSqlGrammarException(BadSqlGrammarException e) {
    log.error("数据库缺失字段!!!!{}", e.getMessage());
    return R.fail("该模块升级中，请10分钟后再访问");
  }

  @ExceptionHandler(SQLSyntaxErrorException.class)
  @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
  public Object sqlSyntaxErrorException(SQLSyntaxErrorException e) {
    log.error("数据库缺失表!!!!{}", e.getMessage());
    return R.fail("该模块升级中，请10分钟后再访问");
  }
}
