package cn.zgfxrc.boot.common.config.exception;

import com.seepine.tool.R;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

/**
 * 处理错误
 *
 * @author seepine
 */
@Slf4j
@AllArgsConstructor
@RestControllerAdvice
public class RestResponseEntityExceptionHandler extends ResponseEntityExceptionHandler {

  /**
   * 用来处理如上错误 @RequestMapping public void test(@RequestParam String a)
   *
   * @param e 异常
   * @param body body
   * @param headers headers
   * @param status status
   * @param request request
   * @return ResponseEntity
   */
  @NotNull
  @Override
  protected ResponseEntity<Object> handleExceptionInternal(
      @NotNull Exception e,
      @Nullable Object body,
      @NotNull HttpHeaders headers,
      @NotNull HttpStatus status,
      @NotNull WebRequest request) {
    if (HttpStatus.INTERNAL_SERVER_ERROR.equals(status)) {
      request.setAttribute("javax.servlet.error.exception", e, 0);
    }
    return new ResponseEntity<>(R.fail(e.getMessage()), headers, HttpStatus.BAD_REQUEST);
  }
}
