package cn.zgfxrc.boot.common.config.runner;

import com.seepine.tool.cache.Cache;
import com.seepine.tool.cache.CacheService;
import com.seepine.tool.lock.Lock;
import com.seepine.tool.lock.LockService;
import java.time.Duration;
import java.util.function.Supplier;
import javax.annotation.Nonnull;
import lombok.AllArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Order(Integer.MIN_VALUE)
@Component
@AllArgsConstructor
public class RedissonInitRunner implements ApplicationRunner {
  RedissonClient redissonClient;

  @Override
  public void run(ApplicationArguments args) throws Exception {
    Cache.enhance(
        new CacheService() {
          @Override
          public void set(@Nonnull String key, @Nonnull Object value, long delayMillisecond) {
            if (delayMillisecond > 0) {
              redissonClient.getBucket(key).set(value, Duration.ofMillis(delayMillisecond));
            } else {
              redissonClient.getBucket(key).set(value);
            }
          }

          @Nullable
          @Override
          public Object get(@Nonnull String key) {
            return redissonClient.getBucket(key).get();
          }

          @Nullable
          @Override
          public Object remove(@Nonnull String key) {
            return redissonClient.getBucket(key).getAndDelete();
          }

          @Override
          public long removeByPattern(@NotNull String pattern) {
            return redissonClient.getKeys().deleteByPattern(pattern);
          }
        });

    Lock.enhance(
        new LockService() {
          @Nullable
          @Override
          public <T> T lock(@Nonnull String key, @Nonnull Supplier<T> supplier) {
            RLock lock = redissonClient.getLock("redisson_lock:" + key);
            try {
              lock.lock();
              return supplier.get();
            } finally {
              lock.unlock();
            }
          }
        });
  }
}
