package cn.zgfxrc.boot.common.core.util;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.seepine.tool.util.ListUtil;
import com.seepine.tool.util.Objects;

import java.util.function.Function;

/**
 * @author huanghs
 * @date 2022.6.20
 */
public class PageUtil {
  /**
   * 分页转换
   *
   * @param originPage 原分页数据
   * @param mapper 转换逻辑
   * @param <T> 原类
   * @param <R> 新类
   * @return 新分页数据
   */
  public static <T, R> Page<R> convertPage(
      IPage<T> originPage, Function<? super T, ? extends R> mapper) {
    if (Objects.isEmpty(originPage)) {
      return new Page<>();
    }
    Page<R> pageRes = new Page<>();
    BeanUtil.copyProperties(originPage, pageRes, "records");
    pageRes.setRecords(ListUtil.map(originPage.getRecords(), mapper));
    return pageRes;
  }

  /**
   * 将IPage转为Page
   *
   * @param originPage iPage
   * @param <T> T
   * @return page
   */
  public static <T> Page<T> convertPage(IPage<T> originPage) {
    if (Objects.isEmpty(originPage)) {
      return new Page<>();
    }
    Page<T> pageRes = new Page<>();
    BeanUtil.copyProperties(originPage, pageRes, "records");
    pageRes.setRecords(originPage.getRecords());
    return pageRes;
  }
}
