package cn.zgfxrc.boot.common.feign;

import cn.zgfxrc.boot.common.core.util.TenantUtil;
import cn.zgfxrc.boot.common.feign.decoder.ResultStatusDecoder;
import com.seepine.secret.AuthUtil;
import com.seepine.secret.entity.AuthUser;
import com.seepine.tool.util.Strings;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import feign.codec.Decoder;
import feign.optionals.OptionalDecoder;
import javax.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.cloud.openfeign.support.ResponseEntityDecoder;
import org.springframework.cloud.openfeign.support.SpringDecoder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

/**
 * 传递请求头和加密
 *
 * @author seepine
 */
@Configuration
public class FeignConfiguration implements RequestInterceptor {

  @Autowired private ObjectFactory<HttpMessageConverters> messageConverters;

  @Bean
  public Decoder feignDecoder() {
    return new ResultStatusDecoder(
        new OptionalDecoder(new ResponseEntityDecoder(new SpringDecoder(messageConverters))));
  }

  @Override
  public void apply(RequestTemplate requestTemplate) {
    ServletRequestAttributes attributes =
        (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
    String authorization = null;
    if (attributes != null) {
      HttpServletRequest request = attributes.getRequest();
      // 传递重要请求头
      requestTemplate.header("token", request.getHeader("token"));
      requestTemplate.header("secret", request.getHeader("secret"));
      authorization = request.getHeader("Authorization");
    }
    if (Strings.isBlank(authorization)) {
      AuthUser user = null;
      try {
        user = AuthUtil.getUser();
      } catch (Exception ignore) {
      }
      if (user != null) {
        if (Strings.nonBlank(user.getToken())) {
          authorization = "Bearer " + user.getToken();
        } else {
          authorization = "Bearer " + AuthUtil.login(user).getToken();
        }
      }
    }
    requestTemplate.header("Authorization", authorization);
    // 添加内部请求头，确保只内部调用
    requestTemplate.header("inner", "true");
    requestTemplate.header("tenantId", TenantUtil.getTenantId());
  }
}
