package cn.zgfxrc.boot.common.feign;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * @author huanghs
 * @date 2022.6.28
 */
public class MultipartFileFactory {
  private static final int BLOCK_SIZE = 8192;
  /**
   * 获取传输的multipartFile，将输入流+文件名转成multipartFile文件，去调用feignClient
   *
   * @param inputStream inputStream
   * @param originalFileName 文件名
   * @param fileParamName 传递给后端的文件param字段
   * @return MultipartFile
   */
  public static MultipartFile getMulFile(
      InputStream inputStream, String originalFileName, String fileParamName) {
    FileItem fileItem = createFileItem(inputStream, originalFileName, fileParamName);
    // CommonsMultipartFile是feign对multipartFile的封装，但是要FileItem类对象
    return new CommonsMultipartFile(fileItem);
  }
  /**
   * 获取传输的multipartFile，将输入流+文件名转成multipartFile文件，去调用feignClient
   *
   * @param inputStream inputStream
   * @param originalFileName 文件
   * @return MultipartFile
   */
  public static MultipartFile getMulFile(InputStream inputStream, String originalFileName) {
    FileItem fileItem = createFileItem(inputStream, originalFileName, "file");
    // CommonsMultipartFile是feign对multipartFile的封装，但是要FileItem类对象
    return new CommonsMultipartFile(fileItem);
  }

  /**
   * FileItem类对象创建
   *
   * @param fis 文件流
   * @param originalFileName 文件名
   * @param fileParamName 传递param
   * @return FileItem
   */
  private static FileItem createFileItem(
      InputStream fis, String originalFileName, String fileParamName) {
    FileItemFactory factory = new DiskFileItemFactory(16, null);
    FileItem item =
        factory.createItem(fileParamName, "multipart/form-data", true, originalFileName);
    int bytesRead;
    byte[] buffer = new byte[BLOCK_SIZE];
    try {
      OutputStream os = item.getOutputStream();
      while ((bytesRead = fis.read(buffer, 0, BLOCK_SIZE)) != -1) {
        os.write(buffer, 0, bytesRead);
      }
      os.close();
      fis.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
    return item;
  }
}
