package cn.zgfxrc.boot.common.feign.decoder;

import cn.zgfxrc.boot.common.feign.FeignWrapException;
import com.fasterxml.jackson.databind.JsonNode;
import com.seepine.json.Json;
import feign.Response;
import feign.codec.ErrorDecoder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.StreamUtils;

import java.nio.charset.Charset;

/**
 * 处理接口异常逻辑
 *
 * @author huanghs
 * @date 2022.7.27
 */
@Slf4j
@Configuration
public class ErrorFallback implements ErrorDecoder {
  @Override
  public Exception decode(String s, Response response) {
    String resultStr;
    try {
      if (response.body() == null) {
        return new FeignWrapException(
            response.status(),
            1,
            response.status() == 401 ? "请先登录" : "",
            response.request(),
            response.headers());
      } else {
        resultStr =
            StreamUtils.copyToString(response.body().asInputStream(), Charset.defaultCharset());
      }
    } catch (Exception e) {
      return new FeignWrapException(
          response.status(), 1, e.getMessage(), response.request(), response.headers());
    }
    JsonNode jsonNode = null;
    try {
      jsonNode = Json.parse(resultStr);
    } catch (Exception ignored) {
    }
    if (jsonNode != null && jsonNode.has("code")) {
      if (jsonNode.get("code").asInt(1) != 0) {
        return new FeignWrapException(
            response.status(),
            jsonNode.get("code").asInt(1),
            jsonNode.get("msg").asText(),
            response.request(),
            response.headers());
      }
    }
    return new FeignWrapException(
        response.status(), 1, resultStr, response.request(), response.headers());
  }
}
