package cn.zgfxrc.boot.common.feign.decoder;

import cn.zgfxrc.boot.common.feign.FeignWrapException;
import com.fasterxml.jackson.databind.JsonNode;
import com.seepine.json.Json;
import feign.Response;
import feign.codec.Decoder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;

import java.io.IOException;
import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.util.Objects;
import java.util.Optional;

/**
 * @author huanghs
 */
@Slf4j
public final class ResultStatusDecoder implements Decoder {
  final Decoder delegate;

  public ResultStatusDecoder(Decoder delegate) {
    Objects.requireNonNull(delegate, "Decoder must not be null. ");
    this.delegate = delegate;
  }

  @Override
  public Object decode(Response response, Type type) throws IOException {
    String contentLength =
        Optional.ofNullable(response.headers().get("content-length"))
            .map(Objects::toString)
            .orElse("");
    if (contentLength.equals("[0]")) {
      return delegate.decode(response, type);
    }
    String contentType =
        Optional.ofNullable(response.headers().get("content-type"))
            .map(Objects::toString)
            .orElse("");
    if (!contentType.contains("json")) {
      return delegate.decode(response, type);
    }
    String resultStr =
        IOUtils.toString(response.body().asInputStream(), StandardCharsets.UTF_8.name());
    JsonNode jsonNode = null;
    try {
      jsonNode = Json.parse(resultStr);
    } catch (Exception ignored) {
    }
    if (jsonNode != null && jsonNode.has("code")) {
      if (jsonNode.get("code").asInt(-1) == 0) {
        if (jsonNode.hasNonNull("data")) {
          resultStr =
              jsonNode.get("data").isTextual()
                  ? jsonNode.get("data").textValue()
                  : jsonNode.get("data").toString();
        } else {
          return null;
        }
      } else {
        resultStr = jsonNode.get("msg").asText();
        throw new FeignWrapException(
            response.status(),
            jsonNode.get("code").asInt(1),
            resultStr,
            response.request(),
            response.headers());
      }
    }
    // 回写body,因为response的流数据只能读一次，这里回写后重新生成response
    return delegate.decode(
        response.toBuilder().body(resultStr, StandardCharsets.UTF_8).build(), type);
  }
}
