package cn.zgfxrc.boot.upms.api.feign;

import cn.zgfxrc.boot.common.feign.MultipartSupportConfig;
import cn.zgfxrc.boot.upms.api.vo.FileInfoVo;
import feign.Response;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.multipart.MultipartFile;

/**
 * upmsTenantFeign
 *
 * @author huanghs
 */
@FeignClient(
    name = "upms-file",
    url = "http://127.0.0.1:${server.port}/sys/file/feign",
    configuration = MultipartSupportConfig.class)
public interface RemoteFileService {
  /**
   * 上传文件
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysFileFeignController#upload
   * @param file 构建文件体
   * @return 返回文件url
   */
  @PostMapping(value = "upload", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
  FileInfoVo upload(@RequestPart("file") MultipartFile file);

  /**
   * 下载文件
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysFileFeignController#download
   * @param filename 文件名
   * @return Response
   */
  @PostMapping(value = "download/{filename}")
  Response download(@PathVariable("filename") String filename);
  /**
   * 下载文件
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysFileFeignController#downloadByUrl
   * @param url 文件地址
   * @return Response
   */
  @PostMapping(value = "download/by/url")
  Response downloadByUrl(@RequestParam("url") String url);
}
