package cn.zgfxrc.boot.upms.api.feign;

import cn.zgfxrc.boot.upms.api.entity.SysTenant;
import cn.zgfxrc.boot.upms.api.entity.req.SysTenantReq;
import cn.zgfxrc.boot.upms.api.entity.res.SysTenantRes;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * upmsTenantFeign
 *
 * @author huanghs
 */
@FeignClient(name = "upms-tenant", url = "http://127.0.0.1:${server.port}/sys/tenant/feign")
public interface RemoteTenantService {
  /**
   * 新增租户，并初始化租户超管信息、角色、菜单等
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysTenantFeignController#add
   * @param entity entity
   * @return 租户信息
   */
  @PostMapping("add")
  SysTenantRes add(SysTenantReq entity);

  /**
   * 编辑
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysTenantFeignController#edit
   * @param entity entity
   * @return true/false
   */
  @PostMapping("edit")
  Boolean edit(SysTenantReq entity);
  /**
   * 删除，谨慎操作
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysTenantFeignController#remove
   * @param id 租户id
   */
  @PostMapping("del/{id}")
  void remove(@PathVariable("id") Long id);
  /**
   * 修改租户信息
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysTenantFeignController#updateName
   * @param id id
   * @param name name
   * @param shortName shortName
   * @return true/false
   */
  @PostMapping("update/name")
  Boolean updateName(
      @RequestParam("id") Long id,
      @RequestParam("name") String name,
      @RequestParam("shortName") String shortName);

  /**
   * 根据租户id获取租户信息，无需登陆
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysTenantFeignController#getById
   * @param id 租户id
   * @return 租户信息
   */
  @PostMapping("get/{id}")
  SysTenant getById(@PathVariable("id") Long id);
}
