package cn.zgfxrc.boot.upms.api.feign;

import cn.zgfxrc.boot.upms.api.entity.SysUser;
import com.seepine.secret.entity.AuthUser;
import java.util.Set;
import javax.annotation.Nonnull;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * upmsTenantFeign
 *
 * @author huanghs
 */
@FeignClient(name = "upms-user", url = "http://127.0.0.1:${server.port}/sys/user/feign")
public interface RemoteUserService {
  /**
   * 修改租户信息
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysUserFeignController#get
   * @param id id
   * @return SysUser
   */
  @PostMapping("get")
  SysUser get(@RequestParam("id") Long id);
  /**
   * 修改租户信息,忽略租户id
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysUserFeignController#getIgnoreTenant
   * @param id id
   * @return SysUser
   */
  @PostMapping("get/ignore/tenant")
  SysUser getIgnoreTenant(@RequestParam("id") Long id);
  /**
   * 获取用户权限集合
   *
   * @see cn.zgfxrc.boot.upms.biz.feign.SysUserFeignController#getPermissions
   * @return permissions
   */
  @Nonnull
  @PostMapping("get/permissions")
  Set<String> getPermissions(AuthUser authUser);
}
