package cn.zgfxrc.boot.upms.biz.controller;

import cn.zgfxrc.boot.common.core.entity.base.DragModel;
import cn.zgfxrc.boot.common.core.entity.interfaces.AddGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.DragGroup;
import cn.zgfxrc.boot.common.core.entity.interfaces.EditGroup;
import cn.zgfxrc.boot.upms.api.entity.SysDept;
import cn.zgfxrc.boot.upms.api.entity.SysDeptUser;
import cn.zgfxrc.boot.upms.api.entity.req.SysDeptReq;
import cn.zgfxrc.boot.upms.api.entity.res.SysDeptRes;
import cn.zgfxrc.boot.upms.api.entity.res.SysDeptTreeNode;
import cn.zgfxrc.boot.upms.api.enums.DeptUserType;
import cn.zgfxrc.boot.upms.biz.service.SysDeptService;
import com.seepine.secret.annotation.Log;
import com.seepine.secret.annotation.Permission;
import com.seepine.tool.util.Validate;
import com.seepine.wrap.WrapException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.Collections;
import java.util.List;
import lombok.AllArgsConstructor;
import org.springframework.lang.Nullable;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@RestController
@Api(tags = "套餐管理")
@RequestMapping("sys/dept")
@AllArgsConstructor
public class SysDeptController {
  SysDeptService service;

  /**
   * 树查询
   *
   * @param entity 实体类
   * @return list
   */
  @ApiOperation("获取树结构")
  @PostMapping("fetch/all")
  public List<SysDeptTreeNode> fetchAll(@RequestBody @Nullable SysDept entity) {
    return service.treeListWithTenant(entity, false, false);
  }
  /**
   * 树查询
   *
   * @return list
   */
  @ApiOperation("获取树结构")
  @PostMapping("fetch/user")
  public List<SysDeptTreeNode> fetchUser() {
    return service.fetchUser();
  }
  /**
   * 树查询,
   *
   * @param entity 实体类
   * @return list
   */
  @ApiOperation("获取树结构")
  @PostMapping("fetch/with/drag")
  @Permission("sys_dept_view")
  public List<SysDeptTreeNode> fetch(@RequestBody @Nullable SysDept entity) {
    return service.treeListWithTenant(entity, true, true);
  }

  /**
   * 拖拽
   *
   * @param req 拖拽数据
   */
  @ApiOperation("拖拽")
  @PostMapping("drag")
  @Permission("sys_dept_edit")
  public void drag(@Validated(DragGroup.class) @RequestBody DragModel req) {
    service.drag(req);
  }
  /**
   * 新增
   *
   * @param entity 实体类
   */
  @Log("新增")
  @ApiOperation("新增")
  @PostMapping("add")
  @Permission("sys_dept_add")
  @Transactional(rollbackFor = Exception.class)
  public void save(@RequestBody @Validated(AddGroup.class) SysDeptReq entity) {
    if (!service.save(entity)) {
      throw new WrapException("保存失败");
    }
    service.resetDeptUser(
        entity.getId(),
        Collections.singletonList(entity.getPrimaryChargeUserId()),
        DeptUserType.PRIMARY);
    service.resetDeptUser(
        entity.getId(), entity.getSecondaryChargeUserIds(), DeptUserType.SECONDARY);
  }

  @Log("修改")
  @ApiOperation("修改")
  @PostMapping("edit/new")
  @Permission("sys_dept_edit")
  @Transactional(rollbackFor = Exception.class)
  public void updateById(@RequestBody @Validated(EditGroup.class) SysDeptReq entity) {
    Validate.isTrue(service.updateById(entity), "保存失败");
    service.resetDeptUser(
        entity.getId(),
        Collections.singletonList(entity.getPrimaryChargeUserId()),
        DeptUserType.PRIMARY);
    service.resetDeptUser(
        entity.getId(), entity.getSecondaryChargeUserIds(), DeptUserType.SECONDARY);
  }

  @Log("删除")
  @ApiOperation("删除")
  @PostMapping("del")
  @Permission("sys_dept_del")
  public void del(@RequestBody SysDept entity) {
    service.del(entity);
  }

  @ApiOperation("通过部门id获取用户ids")
  @PostMapping("user/ids/by/{deptId}")
  public List<Long> userIdsByDeptId(@PathVariable Long deptId) {
    return service.userIdsByDeptId(deptId);
  }

  @ApiOperation("重新设置部门成员")
  @PostMapping("reset/dept/user")
  public void resetDeptUser(@RequestBody SysDeptReq req) {
    service.resetDeptUser(req.getId(), req.getUserIds(), DeptUserType.NORMAL);
  }

  @ApiOperation("移除部门中某个成员")
  @PostMapping("remove/dept/user")
  public void removeDeptUser(@RequestBody SysDeptUser req) {
    service.removeDeptUser(req);
  }

  @ApiOperation("获取部门信息")
  @PostMapping("get")
  public SysDeptRes getById(@RequestBody SysDept req) {
    return service.getResById(req.getId());
  }
}
