package cn.zgfxrc.boot.upms.biz.mapper;

import cn.zgfxrc.boot.common.mybatis.base.MpBaseMapper;
import cn.zgfxrc.boot.upms.api.entity.SysUser;
import cn.zgfxrc.boot.upms.api.enums.DeptUserType;
import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import java.util.List;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * @author huanghs@zgafxrc.cn
 * @date 2021.11.2
 */
@Mapper
public interface SysUserMapper extends MpBaseMapper<SysUser> {
  /**
   * 用于登录时，根据手机号密码忽略租户查询
   *
   * @param phone 手机号
   * @param password 密码
   * @return PartnerUser
   */
  @InterceptorIgnore(tenantLine = "true")
  @Select("select * from sys_user where phone=#{phone} and password=#{password} and is_delete=0")
  List<SysUser> listWithoutTenantId(
      @Param("phone") String phone, @Param("password") String password);

  /**
   * 根据id查询租户查询
   *
   * @param id id
   * @return SysUser
   */
  @InterceptorIgnore(tenantLine = "true")
  @Select("select * from sys_user where id=#{id} and is_delete=0")
  SysUser getByIdWithoutTenantId(@Param("id") Long id);
  /**
   * 特殊分页查询，可查询部门
   *
   * @param deptIds 部门ids
   * @param tenantId 租户id
   * @param lastSql 最后查询sql(注意避免sql注入)
   * @return pageRes
   */
  @InterceptorIgnore(tenantLine = "true")
  @Select(
      "SELECT COUNT(*) FROM (select u.* from sys_dept_user as du left JOIN sys_user as u on u.id = du.user_id where u.is_delete=0 and du.dept_id in ${deptIds} and du.tenant_id=#{tenantId} ${lastSql} GROUP BY u.id) TOTAL")
  int count(
      @Param("deptIds") String deptIds,
      @Param("tenantId") Long tenantId,
      @Param("lastSql") String lastSql);
  /**
   * 特殊分页查询，可查询部门
   *
   * @param deptIds 部门ids
   * @param tenantId 租户id
   * @param lastSql 最后查询sql(注意避免sql注入)
   * @return pageRes
   */
  @InterceptorIgnore(tenantLine = "true")
  @Select(
      "select u.* from sys_dept_user as du left JOIN sys_user as u on u.id = du.user_id"
          + " where u.is_delete=0 and du.is_delete=0 and du.dept_id in ${deptIds} and du.tenant_id=#{tenantId} and du.type=#{deptUserType} ${lastSql} GROUP BY u.id limit #{current},#{size}")
  List<SysUser> page(
      @Param("deptIds") String deptIds,
      @Param("tenantId") Long tenantId,
      @Param("deptUserType") DeptUserType deptUserType,
      @Param("lastSql") String lastSql,
      @Param("current") long current,
      @Param("size") long size);
}
