package cn.zgfxrc.boot.upms.biz.service;

import com.anji.captcha.service.CaptchaCacheService;
import com.seepine.tool.cache.Cache;
import org.springframework.stereotype.Service;

/**
 * 对于分布式部署的应用，我们建议应用自己实现CaptchaCacheService，比如用Redis，参考service/spring-boot代码示例。
 * 如果应用是单点的，也没有使用redis，那默认使用内存。 内存缓存只适合单节点部署的应用，否则验证码生产与验证在节点之间信息不同步，导致失败。
 *
 * <p>☆☆☆ SPI： 在resources目录新建META-INF.services文件夹(两层)，参考当前服务resources。 @Title: 使用redis缓存
 *
 * @author lide1202@hotmail.com
 * @date 2020-05-12
 */
@Service
public class CaptchaCacheServiceRedisImpl implements CaptchaCacheService {

  @Override
  public String type() {
    return "redis";
  }

  @Override
  public void set(String key, String value, long expiresInSeconds) {
    Cache.set(key, value, expiresInSeconds == 0 ? 60 * 60 * 1000 : expiresInSeconds * 1000);
  }

  @Override
  public boolean exists(String key) {
    return Cache.get(key) != null;
  }

  @Override
  public void delete(String key) {
    Cache.remove(key);
  }

  @Override
  public String get(String key) {
    return Cache.getStr(key);
  }

  @Override
  public Long increment(String key, long val) {
    Long ret = Long.parseLong(get(key)) + val;
    set(key, ret + "", 0);
    return ret;
  }
}
