package cn.zgfxrc.boot.upms.biz.service;

import cn.zgfxrc.boot.common.core.constant.Constant;
import cn.zgfxrc.boot.common.core.entity.base.BaseModel;
import cn.zgfxrc.boot.common.core.entity.base.BaseModelWithTenant;
import cn.zgfxrc.boot.common.core.enums.PlatformType;
import cn.zgfxrc.boot.common.core.util.TenantUtil;
import cn.zgfxrc.boot.upms.api.entity.SysDict;
import cn.zgfxrc.boot.upms.api.entity.SysDictItem;
import cn.zgfxrc.boot.upms.api.enums.DictType;
import cn.zgfxrc.boot.upms.biz.mapper.SysDictItemMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysDictMapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.seepine.secret.AuthUtil;
import com.seepine.tool.lock.Lock;
import com.seepine.tool.util.Validate;
import java.util.*;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author huanghs
 * @date 2022.12.2
 */
@Service
@AllArgsConstructor
@Transactional(rollbackFor = Exception.class)
public class SysDictItemService extends ServiceImpl<SysDictItemMapper, SysDictItem> {
  private final SysDictMapper sysDictMapper;

  @Override
  public boolean save(SysDictItem entity) {
    Validate.nonBlank(entity.getLabel(), "字典标签不能为空");
    Validate.nonBlank(entity.getValue(), "字典值不能为空");
    return Lock.sync(
        Validate.nonNull(entity.getDictId(), "字典id不能为空"),
        () -> {
          entity.setSort(
              Optional.ofNullable(
                          super.lambdaQuery()
                              .select(SysDictItem::getSort)
                              .eq(SysDictItem::getDictId, entity.getDictId())
                              .orderByDesc(SysDictItem::getSort)
                              .last("limit 1")
                              .one())
                      .map(SysDictItem::getSort)
                      .orElse(-1)
                  + 1);
          Validate.isTrue(
              super.lambdaQuery()
                      .eq(SysDictItem::getValue, entity.getValue())
                      .eq(SysDictItem::getDictId, entity.getDictId())
                      .count()
                  == 0,
              "字典值已存在");
          Validate.isTrue(
              super.lambdaQuery()
                      .eq(SysDictItem::getLabel, entity.getLabel())
                      .eq(SysDictItem::getDictId, entity.getDictId())
                      .count()
                  == 0,
              "字典标签已存在");
          var dict = sysDictMapper.lambdaQuery().eq(BaseModel::getId, entity.getDictId()).one();
          // 如果是平台端，且是为默认字典新增字典项时，为已存在租户也插入值
          if (dict.getType().equals(DictType.DEFAULT)
              && PlatformType.SYS.eq(
                  PlatformType.from(AuthUtil.getUser().getClaim(Constant.platformType)))) {
            for (var sysDict :
                sysDictMapper.getListIgnoreTenant(
                    Wrappers.<SysDict>lambdaQuery()
                        .eq(SysDict::getDicCode, dict.getDicCode())
                        .ne(BaseModelWithTenant::getTenantId, AuthUtil.getUser().getTenantId()))) {
              TenantUtil.run(
                  sysDict.getTenantId(),
                  () -> {
                    var item = new SysDictItem();
                    BeanUtils.copyProperties(entity, item);
                    item.setDictId(sysDict.getId());
                    item.setSort(
                        Optional.ofNullable(
                                    super.lambdaQuery()
                                        .select(SysDictItem::getSort)
                                        .eq(SysDictItem::getDictId, item.getDictId())
                                        .orderByDesc(SysDictItem::getSort)
                                        .last("limit 1")
                                        .one())
                                .map(SysDictItem::getSort)
                                .orElse(-1)
                            + 1);
                    if (super.lambdaQuery()
                                .eq(SysDictItem::getValue, item.getValue())
                                .eq(SysDictItem::getDictId, item.getDictId())
                                .count()
                            == 0
                        && super.lambdaQuery()
                                .eq(SysDictItem::getLabel, item.getLabel())
                                .eq(SysDictItem::getDictId, item.getDictId())
                                .count()
                            == 0) {
                      Validate.isTrue(super.save(item), "保存失败");
                    }
                  });
            }
          }
          return super.save(entity);
        });
  }

  public List<SysDictItem> list(Long dictId) {
    return lambdaQuery()
        .eq(SysDictItem::getDictId, dictId)
        .eq(SysDictItem::getDisabled, false)
        .list();
  }
}
