package cn.zgfxrc.boot.upms.biz.service;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.zgfxrc.boot.common.core.constant.Constant;
import cn.zgfxrc.boot.common.core.entity.base.BaseModel;
import cn.zgfxrc.boot.common.core.enums.ActiveStatus;
import cn.zgfxrc.boot.common.core.enums.PlatformType;
import cn.zgfxrc.boot.common.core.util.PageUtil;
import cn.zgfxrc.boot.common.core.util.TenantUtil;
import cn.zgfxrc.boot.upms.api.entity.*;
import cn.zgfxrc.boot.upms.api.entity.req.SysUserReq;
import cn.zgfxrc.boot.upms.api.entity.res.SysUserRes;
import cn.zgfxrc.boot.upms.api.enums.DeptUserType;
import cn.zgfxrc.boot.upms.api.enums.PositionType;
import cn.zgfxrc.boot.upms.biz.mapper.SysTenantMapper;
import cn.zgfxrc.boot.upms.biz.mapper.SysUserMapper;
import cn.zgfxrc.boot.upms.biz.util.SecureUtil;
import cn.zgfxrc.boot.upms.biz.util.ValidUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.seepine.secret.AuthUtil;
import com.seepine.secret.entity.AuthUser;
import com.seepine.secret.entity.Platform;
import com.seepine.tool.Run;
import com.seepine.tool.lock.Lock;
import com.seepine.tool.util.ListUtil;
import com.seepine.tool.util.LocalDateUtil;
import com.seepine.tool.util.Objects;
import com.seepine.tool.util.Validate;
import com.seepine.wrap.WrapException;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.Serializable;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author huanghs@zgfxrc.cn
 * @date 2021.11.2
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysUserService extends ServiceImpl<SysUserMapper, SysUser> {
  private final SysMenuService sysMenuService;
  private final SysRoleService sysRoleService;
  private final SysTenantMapper sysTenantMapper;

  private final SysDeptService sysDeptService;
  private final SysPositionService sysPositionService;
  private final SysPositionUserService sysPositionUserService;
  /**
   * 分页扩展
   *
   * @param page page
   * @param entity entity
   * @return IPage
   * @date 2021.11.9 huanghs@zgfxrc.cn
   */
  public IPage<SysUserRes> getPageRes(Page<SysUser> page, @Nullable SysUserReq entity) {
    if (entity == null) {
      entity = new SysUserReq();
    }
    // 如果部门为空
    if (entity.getDeptId() == null) {
      LambdaQueryWrapper<SysUser> wrapper = Wrappers.lambdaQuery();
      Run.nonBlank(entity.getPhone(), val -> wrapper.like(SysUser::getPhone, val));
      Run.nonBlank(entity.getFullName(), val -> wrapper.like(SysUser::getFullName, val));
      Run.nonBlank(entity.getEmail(), val -> wrapper.like(SysUser::getEmail, val));
      Run.nonEmpty(entity.getStatus(), val -> wrapper.eq(SysUser::getStatus, val));
      return PageUtil.convertPage(super.page(page, wrapper), this::mapper);
    }
    AtomicReference<String> lastSql = new AtomicReference<>(" and 1=1 ");
    Run.nonBlank(
        entity.getPhone(),
        val -> lastSql.set(lastSql.get() + " and u.phone like concat('%','" + val + "','%')"));
    Run.nonBlank(
        entity.getFullName(),
        val -> lastSql.set(lastSql.get() + " and u.full_name like concat('%','" + val + "','%')"));
    Run.nonBlank(
        entity.getEmail(),
        val -> lastSql.set(lastSql.get() + " and u.email like concat('%','" + val + "','%')"));
    Run.nonEmpty(
        entity.getStatus(), val -> lastSql.set(lastSql.get() + " and u.status like '" + val + "'"));
    AuthUser authUser = AuthUtil.getUser();
    String deptIdsSql =
        "("
            + (Boolean.FALSE.equals(entity.getInnerOnly())
                    ? sysDeptService.findAllChildrenIds(entity.getDeptId())
                    : Collections.singletonList(entity.getDeptId()))
                .stream().map(Object::toString).collect(Collectors.joining(","))
            + ")";
    List<SysUserRes> records =
        ListUtil.map(
            baseMapper.page(
                deptIdsSql,
                authUser.getTenantIdAsLong(),
                DeptUserType.NORMAL,
                lastSql.get(),
                page.getCurrent() - 1,
                page.getSize()),
            this::mapper);
    Page<SysUserRes> pageRes = new Page<>();
    pageRes.setCurrent(page.getCurrent());
    pageRes.setSize(page.getSize());
    pageRes.setTotal(baseMapper.count(deptIdsSql, authUser.getTenantIdAsLong(), lastSql.get()));
    pageRes.setRecords(records);
    return pageRes;
  }

  private SysUserRes mapper(SysUser item) {
    SysUserRes res = new SysUserRes();
    BeanUtils.copyProperties(item, res);
    res.setRoleIds(sysRoleService.getRoleIds(item.getId()));
    // 获取用户所属部门关系
    Run.nonEmpty(
        sysDeptService.getDeptUsers(item.getId()),
        (deptUsers) -> {
          // 设置用户部门ids
          res.setDeptIds(
              deptUsers.stream().map(SysDeptUser::getDeptId).collect(Collectors.toList()));
          // 获取关联的部门名称list
          if (Objects.nonEmpty(res.getDeptIds())) {
            List<String> deptName =
                res.getDeptIds().stream()
                    .map(
                        w -> {
                          var sysDept =
                              sysDeptService
                                  .getBaseMapper()
                                  .lambdaQuery()
                                  .eq(SysDept::getId, w)
                                  .one();
                          return Objects.nonNull(sysDept) && Objects.nonBlank(sysDept.getName())
                              ? sysDept.getName()
                              : "";
                        })
                    .filter(Objects::nonBlank)
                    .toList();
            if (Objects.nonEmpty(deptName)) {
              res.setDeptNames(String.join(",", deptName));
            }
            var positionUser =
                sysPositionUserService
                    .lambdaQuery()
                    .select(SysPositionUser::getPositionId)
                    .eq(SysPositionUser::getUserId, item.getId())
                    .eq(SysPositionUser::getType, PositionType.PRIMARY)
                    .last(" limit 1")
                    .one();
            if (Objects.nonNull(positionUser)) {
              res.setPositionPrimaryId(positionUser.getPositionId());
            }
            var positionIds =
                sysPositionUserService
                    .lambdaQuery()
                    .select(SysPositionUser::getPositionId)
                    .eq(SysPositionUser::getUserId, item.getId())
                    .eq(SysPositionUser::getType, PositionType.SECONDARY)
                    .list()
                    .stream()
                    .map(SysPositionUser::getPositionId)
                    .toList();
            res.setPositionSecondaryIds(positionIds);
          }
        });
    res.setPassword(null);
    return res;
  }
  /**
   * 修改超管密码
   *
   * @param tenantId 租户id
   * @param newPassword 新密码
   */
  public void updateAdminPassword(Long tenantId, String newPassword) {
    TenantUtil.run(
        tenantId,
        () -> {
          if (!StringUtils.hasText(newPassword)) {
            return;
          }
          ValidUtil.validPassword(newPassword);
          Validate.isTrue(
              lambdaUpdate()
                  .eq(SysUser::getIsSystem, true)
                  .set(SysUser::getPassword, SecureUtil.encrypt(newPassword.trim()))
                  .update(),
              "修改超管密码失败");
        });
  }

  /**
   * 一般用于第一次新增用户，其他慎用
   *
   * @param entity entity
   * @date 2021.11.15 huanghs
   * @date 2022.12.13 huanghs 增加status和用户数判断
   */
  @Transactional(rollbackFor = Exception.class)
  public void add(SysUserReq entity) throws WrapException {
    ValidUtil.validPhone(Validate.nonBlank(entity.getPhone(), "手机号不能为空").replace(" ", "").trim());
    ValidUtil.validPassword(entity.getPassword());
    if (entity.getPlatformType() == null) {
      AuthUser authUser = AuthUtil.getUser();
      entity.setPlatformType(PlatformType.from(authUser.getClaim("platformType")));
    }
    Run.isEmpty(entity.getStatus(), () -> entity.setStatus(ActiveStatus.NORMAL));
    // 如果状态是正常，判断有没有超员
    if (entity.getStatus().equals(ActiveStatus.NORMAL)) {
      SysTenant sysTenant = sysTenantMapper.selectById(TenantUtil.getTenantId());
      if (sysTenant.getActiveUserNumberLimit() > 0) {
        Validate.isTrue(
            lambdaQuery().eq(SysUser::getStatus, ActiveStatus.NORMAL).count()
                < sysTenant.getActiveUserNumberLimit(),
            "活跃成员数已达上限，请锁定其他成员后再新增，或联系平台管理员");
      }
    }
    Lock.sync(
        entity.getPhone(),
        () -> {
          Run.isEmpty(entity.getIsSystem(), () -> entity.setIsSystem(false));
          entity.setFullName(entity.getFullName().trim());
          Assert.isNull(getByPhone(entity.getPhone()), "手机号已被注册");
          entity.setPassword(SecureUtil.encrypt(entity.getPassword()));
          Assert.isTrue(super.save(entity), "保存失败");
          AuthUtil.execute(
              AuthUser.builder()
                  .id(entity.getId())
                  .fullName(entity.getFullName())
                  .tenantId(TenantUtil.getTenantId())
                  .build(),
              () -> {
                sysRoleService.saveUserRoles(entity.getId(), entity.getRoleIds());
                // 设置部门关联
                sysDeptService.resetUserDept(entity.getId(), entity.getDeptIds());
                // 设置职位管理
                sysPositionService.savePositionUser(
                    entity.getId(),
                    entity.getPositionSecondaryIds(),
                    entity.getPositionPrimaryId());
              });
        });
  }

  /**
   * 一般用于管理员修改用户信息，其他慎用
   *
   * @param entity entity
   */
  @Transactional(rollbackFor = Exception.class)
  public void editInfo(SysUserReq entity) throws WrapException {
    String newPwd = null;
    if (Objects.nonBlank(entity.getPassword())) {
      ValidUtil.validPassword(entity.getPassword());
      newPwd = SecureUtil.encrypt(entity.getPassword());
    }
    Validate.isTrue(
        super.lambdaUpdate()
            .eq(BaseModel::getId, entity.getId())
            .set(SysUser::getFullName, entity.getFullName())
            .set(SysUser::getAvatarUrl, entity.getAvatarUrl())
            .set(SysUser::getPhone, entity.getPhone())
            .set(SysUser::getEmail, entity.getEmail())
            .set(SysUser::getLeaderId, entity.getLeaderId())
            .set(SysUser::getStatus, entity.getStatus())
            .set(Objects.nonBlank(newPwd), SysUser::getPassword, newPwd)
            .update(),
        "保存失败");
    // 如果状态是正常，判断有没有超员
    if (entity.getStatus().equals(ActiveStatus.NORMAL)) {
      SysTenant sysTenant =
          Validate.nonEmpty(sysTenantMapper.selectById(TenantUtil.getTenantId()), "获取租户信息失败，请重新登录");
      if (sysTenant.getActiveUserNumberLimit() > 0) {
        Validate.isTrue(
            lambdaQuery().eq(SysUser::getStatus, ActiveStatus.NORMAL).count()
                < sysTenant.getActiveUserNumberLimit(),
            "活跃成员数已达上限，请锁定其他成员后再新增，或联系平台管理员");
      }
    }
    // 重置部门关联
    sysDeptService.resetUserDept(entity.getId(), entity.getDeptIds());
    // 重置角色关联
    sysRoleService.resetUserRoles(entity.getId(), entity.getRoleIds());
    // 重置职位关联
    sysPositionService.savePositionUser(
        entity.getId(), entity.getPositionSecondaryIds(), entity.getPositionPrimaryId());
  }

  @SneakyThrows
  @Override
  public boolean removeById(Serializable id) {
    SysUser entity = super.getById(id);
    if (entity != null && entity.getIsSystem()) {
      throw new WrapException("系统内置用户无法删除");
    }
    return super.removeById(id);
  }

  public List<AuthUser> login(String phone, String password) {
    return login(baseMapper.listWithoutTenantId(phone, SecureUtil.encrypt(password)));
  }

  public List<AuthUser> login(List<SysUser> list) {
    return login(list, 24 * 60 * 60);
  }

  public List<AuthUser> login(List<SysUser> list, Integer expiresSecond) {
    if (list == null) {
      return null;
    }
    List<AuthUser> result = new ArrayList<>();
    for (SysUser find : list) {
      // 1. 获取租户信息并判断是否锁定
      SysTenant tenant = sysTenantMapper.selectById(find.getTenantId());
      if (tenant == null) {
        continue;
      }
      ActiveStatus status;
      // 1.1 租户过期
      if (LocalDateUtil.isLt(tenant.getExpirationDate(), LocalDate.now())) {
        status = ActiveStatus.SUSPEND;
      }
      // 1.2 租户锁定
      else if (tenant.getIsLock()) {
        status = ActiveStatus.SUSPEND;
      }
      // 1.3 赋值为用户的状态
      else {
        status = find.getStatus();
      }
      AuthUser res =
          AuthUser.builder()
              .id(find.getId())
              .fullName(find.getFullName())
              .phone(find.getPhone())
              .email(find.getEmail())
              .withClaim(Constant.isSystem, find.getIsSystem())
              .withClaim(Constant.status, status.getValue())
              .withClaim(Constant.platformType, find.getPlatformType().getValue())
              .withClaim(Constant.tenantLogo, tenant.getLogo())
              .withClaim(Constant.tenantShortName, tenant.getShortName())
              .withClaim(Constant.tenantParentId, tenant.getParentId())
              .roles(Stream.of(find.getPlatformType().getValue()).collect(Collectors.toSet()))
              .platform(Platform.PC)
              .tenantId(find.getTenantId())
              .tenantName(tenant.getName())
              .build();
      // 增加快捷租户id获取
      if (find.getPlatformType().eq(PlatformType.SYS)) {
        res.withClaim(Constant.sysId, find.getTenantId().toString());
      } else if (find.getPlatformType().eq(PlatformType.ENTERPRISE)) {
        res.withClaim(Constant.enterpriseId, find.getTenantId().toString());
      }
      result.add(AuthUtil.login(res, expiresSecond));
    }
    return result;
  }

  public SysUser getByPhone(String phone) {
    return StrUtil.isBlank(phone) ? null : lambdaQuery().eq(SysUser::getPhone, phone).one();
  }
}
